#include <stdio.h>
#include <stdlib.h>
#include "catread.h"
int catfrq(molec,cfreq,buff)
int molec;
char *cfreq, *buff;
{
/*****************************************************************
C
C     CATFRQ CALLS
C       CATDIR
C         CALLS  CATRD
C       CATRD
C         CALLS CATFIL
C
C   FUNCTION TO FIND AND RETURN FIRST LINE FOR SPECIES "MOLEC"
C       THAT IS BEYOND FREQ
C
C   CATFRQ <0 IF ERROR FOUND
C   CATFRQ =0 IF FREQ GT FREQUENCY OF LAST LINE
C   CATFRQ >0 POINTS TO LINE 
C
C   BUFF CONTAINS THE LINE INFORMATION
C
C   FREQ IS CHARACTER STRING IN FORMAT (F13.4) OR EQUIVALENT
C
******************************************************************/
  int line,nline,l,r;
  nline=catlen(molec);   
/*     lines  < L have frequency  < FREQ */
/*     lines >= R have frequency >= FREQ */
  l=1; r=nline+1; buff[0]=0;
  while(l<r) {
    line= (l + r) >>1;
    if(catrd(molec,line,buff)) return (-1);
    if(strcmp(cfreq,buff) >=0){
      l=line+1;
    }else{
      r=line;
    }
  }
  if(l>nline) {
    buff[0]=0; return (0);
  }
  if(r==line) return (line);
  line=r;
  if(catrd(molec,line,buff)) return (-1);
  return (line);
}

int catrd(molec,line,buff)
int molec, line;
char* buff;
/************************************************************************
C
C     CATRD CALLS
C       CATFIL
C
C
C     SUBROUTINE TO READ CATALOGUE FILE FOR MOLECULE "MOLEC",LINE# "LINE"
C
C       80 CHARACTER BUFFER RETURNED IN BUFF
C
C     ERROR CODE RETURNED (IOSTAT ERROR #)
C
*************************************************************************/
{
#include <fcntl.h>
      static int buflen=80;
      static int omolec=-1;
      static int handle;
      char *catfil();
      long offset;
      if(molec-omolec) { /* new molecule */
        if(omolec>0) close(handle);
        omolec=molec; 
        handle=open(catfil(molec),O_RDONLY);
        if(handle==-1) {
          buff[0]=0; omolec=-1; 
          return (-1);
        }
      }
      if(line<=0) return(-2);
      offset=(line-1)*(long)(buflen+1);
      lseek(handle,offset,0);
      if(read(handle,buff,buflen)==buflen) {
        buff[buflen]=0;
        return (0);
      }else{
        buff[0]=0;
        return (1);
      }
}

char *catfil(num)
int num;
/**********************************************************
     FUNCTION WHICH RETURNS FILE NAME CORRESPONDING TO NUM
***********************************************************/
{
   static char catdir[]={"/catalog/catdir.cat"};
   static char catent[]={"/catalog/c000000.cat"}; 
   int i,k,ii,iq;
   if(num==0) return catdir;
   for(i=0;catent[i]!='.';i++);
   for(k=0; k<6; k++){
      iq=num; num=num/10;
      catent[--i] = '0'+ iq - num*10;
   }
   return catent;
}
#define NSPC 512
static int cat_nmol,cat_lposn;
static struct {
  int moltag,nlen,ver;
  float qln[7];
  char cname[16];} catcom[NSPC], *catptr;
  
int catlen(molec)
int molec;
/**************************************************
C
C   SUBROUTINE TO RETURN CATALOGUE ENTRY LENGTH
C
C   MOLEC IS SPECIES TAG
C
****************************************************/
{
  FILE *cdir;
  char *catfil(), buf[82], *qbuf, ch;
  float *qptr;
  int k,m,n;
  if(cat_nmol==0) {
    cdir=fopen(catfil(0),"r");
    if(cdir==0) return (0);
    catptr=catcom;
    while(cat_nmol<NSPC) {
      if(fgets(buf,82,cdir)==0) break;
      qbuf=catptr->cname;
      for(k=0;k<15;k++) qbuf[k]=buf[k+6];
      buf[6]=0; qbuf[15]=0;
      catptr->moltag=atoi(buf);
      qbuf=buf+26; 
      ch=qbuf[0]; qbuf[0]=0;
      catptr->nlen=atoi(buf+21);
      qbuf[0]=ch; qptr=catptr->qln;
      for(k=0; k<7; k++) {
        ch=qbuf[7]; qbuf[7]=0;
        qptr[k]=atof(qbuf);
        qbuf[7]=ch; qbuf += 7;
      }
      qbuf[2]=0;
      catptr->ver=atoi(qbuf); cat_nmol++; catptr++;
    }
    cat_lposn=0;
  }
  if(molec>0) {
    if(molec==catcom[cat_lposn].moltag) return catcom[cat_lposn].nlen;
    for(cat_lposn=0; cat_lposn<cat_nmol; cat_lposn++) 
      if(molec==catcom[cat_lposn].moltag) return catcom[cat_lposn].nlen;
  }
  cat_lposn=0;
  return 0;
}

char *catdir(molec,nline,qqln,iver)
int molec, *nline, *iver;
float *qqln;
/*********************************************************
C   SUBROUTINE TO RETURN CATALOGUE DIRECTORY INFORMATION
C
C   MOLEC IS SPECIES TAG
C   CATDIR IS ASCII SPECIES NAME
C   NLINE IS THE NUMBER OF LINES FOR MOLECULE
C   QLN IS THE LOG10 OF THE PARTITION FUNCTION FOR
C        300,225,150,75,37.5,18.75,9.375 K
C   IVER IS THE VERSION NUMBER
*************************************************************/
{
  int k;
  static char cerr[]={"err"};  
  float *qptr;
  *nline=catlen(molec);
  if(*nline==0) return cerr;
  catptr=catcom+cat_lposn;
  qptr=catptr->qln;
  for(k=0; k<7; k++)
    qqln[k]=qptr[k];
  *iver=catptr->ver;
  return catptr->cname;
}

int nxtdir(molec)
int *molec;
/**********************************************************************
C
C     FUNCTION NXTDIR RETURNS THE NUMBER OF REMAINING DIRECTORY ENTRIES
C        AFTER INPUT SPECIES MOLEC
C     ON RETURN MOLEC IS CHANGED TO THE NEXT SPECIES TAG
C
C     IF INPUT SPECIES MOLEC = 0 THEN POSITION IS BEFORE FIRST ENTRY
C
C     IF INPUT SPECIES MOLEC = LAST ENTRY THEN MOLEC=0 ON RETURN
***********************************************************************/
{
  catlen(*molec);
  if(*molec) {
    cat_lposn++;
    if(cat_lposn >= cat_nmol) {
       cat_lposn=0; *molec=0; return 0;
    }
  }
  *molec=catcom[cat_lposn].moltag;
  return (cat_nmol-cat_lposn);
}
